import requests

from formaloo import constants, client


class CustomerBatch:

    def __init__(self, customers=[]):
        self.client = client.Client()
        self.customers = customers

    def add_customer(self, customer):
        self.customers.append(customer)

    def create(self):
        headers = self.client.get_headers()
        body = self.get_body()

        response = requests.post(
            constants.V_1_0_CUSTOMER_BATCH_ENDPOINT,
            headers=headers,
            data=body
        )

        return response

    def get_body(self):
        body = []

        for customer in self.customers:
            body.append(
                customer.get_body()
            )

        return body



from datetime import timedelta, datetime

import requests

from . import constants
from .settings import CLIENT_SECRET, CLIENT_KEY


class Client:
    def __init__(self):
        self.authorization_token = None
        self.AUTHORIZATION_EXPIRY_TIME = None

    def get_headers(self):
        headers = self.get_application_header()

        headers.update(self.get_authorization_header())

        return headers

    def get_application_header(self):
        headers = {
            constants.APPLICATION_HEADER: CLIENT_KEY
        }

        return headers

    def get_authorization_header(self):
        headers = {}

        if CLIENT_SECRET:
            if (
                not self.authorization_token or
                self.authorization_expiry_time < datetime.now()
            ):
                self.obtain_authorization_token()

            headers[constants.AUTHORIZATION_HEADER] = "{bearer} {token}".format(
                bearer=constants.AUTHORIZATION_BEARER,
                token=self.authorization_token
            )

        return headers

    def obtain_authorization_token(self):
        if CLIENT_SECRET:
            request_headers = self.get_application_header()

            request_headers[constants.AUTHORIZATION_HEADER] = "{bearer} {token}".format(
                bearer=constants.CREDENTIAL_BEARER,
                token=CLIENT_SECRET
            )

            request_body = {
                'grant_type': constants.CREDENTIAL_GRAT_TYPE
            }

            response = requests.post(
                constants.V_1_0_AUTHORIZATION_TOKEN_ENDPOINT,
                headers=request_headers,
                data=request_body
            )

            if response.status_code == 200:
                self.authorization_token = response.json().get('authorization_token')
                self.authorization_expiry_time = datetime.now() + timedelta(
                    seconds=constants.AUTHORIZATION_TOKEN_TIMEOUT
                )

            elif response.status_code in [400, 401, 403]:
                # TODO handle errors properly
                raise Exception(response.json().get('errors'))



import requests

from formaloo import constants, client


class Customer:

    def __init__(self, base_data={}, extra_data={}, tags=[], **kwargs):
        self.client = client.Client()
        self.base_customer_data = base_data
        self.extra_customer_data = extra_data
        self.tags = tags

        for key, value in kwargs.items():
            if key in constants.CUSTOMER_BASE_FIELDS:
                self.base_customer_data[key] = value
            else:
                self.extra_customer_data[key] = value

    def create(self):
        headers = self.client.get_headers()
        body = self.get_body()

        response = requests.post(
            constants.V_1_0_CREATE_CUSTOMER_ENDPOINT,
            headers=headers,
            data=body
        )

        return response

    def get_body(self):
        body = {
            'customer_data': self.extra_customer_data,
            'tags': self.tags
        }
        body.update(self.base_customer_data)

        return body



from environs import Env

env = Env()
env.read_env()

try:
    from django.conf import settings

    V_1_0_API_BASE = getattr(settings, "FORMALOO_API_BASE", None)
    CLIENT_KEY = getattr(settings, 'FORMALOO_CLIENT_KEY', None)
    CLIENT_SECRET = getattr(settings, 'FORMALOO_CLIENT_SECRET', None)

except ModuleNotFoundError:
    V_1_0_API_BASE = None
    CLIENT_KEY = None
    CLIENT_SECRET = None


if not V_1_0_API_BASE:
    V_1_0_API_BASE = env("FORMALOO_API_BASE", 'https://api.formaloo.net/v1.0/')

if not CLIENT_KEY:
    CLIENT_KEY = env('FORMALOO_CLIENT_KEY')

if not CLIENT_SECRET:
    CLIENT_SECRET = env('FORMALOO_CLIENT_SECRET', None)



import requests

from formaloo import constants, client


class Activity:

    def __init__(self, action, customer_data, activity_data={}, activity_date=None, tags=[]):
        self.client = client.Client()
        self.action = action
        self.customer_data = customer_data
        self.activity_data = activity_data
        self.activity_date = activity_date
        self.tags = tags

    def create(self):
        headers = self.client.get_headers()
        body = self.get_body()

        response = requests.post(
            constants.V_1_0_CREATE_ACTIVITY_ENDPOINT,
            headers=headers,
            data=body
        )

        return response

    def get_body(self):
        body = {
            'action': self.action,
            'custmer': self.customer_data,
            'activity_data': self.activity_data,
            'activity_date': self.activity_date,
            'tags': self.tags
        }

        return body



import requests

from formaloo import constants, client


class ActivityBatch:

    def __init__(self, activities=[]):
        self.client = client.Client()
        self.activities = activities

    def add_activity(self, activity):
        self.activities.append(activity)

    def create(self):
        headers = self.client.get_headers()
        body = self.get_body()

        response = requests.post(
            constants.V_1_0_ACTIVITY_BATCH_ENDPOINT,
            headers=headers,
            data=body
        )

        return response

    def get_body(self):
        body = []

        for activity in self.activities:
            body.append(
                activity.get_body()
            )

        return body
